"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "getJSONSchemaModel", {
    enumerable: true,
    get: function() {
        return getJSONSchemaModel;
    }
});
const _properties = require("./properties");
function getRelationScalarFields(model) {
    return model.fields.flatMap((field)=>field.relationFromFields || []);
}
const getRelationFieldNames = (model)=>{
    return model.fields.filter((field)=>field.relationFromFields || field.relationToFields).map((field)=>field.name);
};
function getJSONSchemaModel(modelMetaData, transformOptions) {
    return (model)=>{
        const definitionPropsMap = model.fields.map((0, _properties.getJSONSchemaProperty)(modelMetaData, transformOptions));
        const propertiesMap = definitionPropsMap.map(([name, definition])=>[
                name,
                definition
            ]);
        const relationScalarFields = getRelationScalarFields(model);
        const relationFieldNames = getRelationFieldNames(model);
        const propertiesWithoutRelationScalars = propertiesMap.filter((prop)=>!relationScalarFields.includes(prop[0]));
        const definition = {
            type: "object",
            properties: {}
        };
        if (transformOptions.keepRelationScalarFields === "true") {
            if (transformOptions.keepRelationFields === "false") {
                definition.properties = Object.fromEntries(propertiesMap.filter((prop)=>!relationFieldNames.includes(prop[0])));
            } else {
                definition.properties = Object.fromEntries(propertiesMap);
            }
        } else {
            if (transformOptions.keepRelationFields === "false") {
                definition.properties = Object.fromEntries(propertiesWithoutRelationScalars.filter((prop)=>!relationFieldNames.includes(prop[0])));
            } else {
                definition.properties = Object.fromEntries(propertiesWithoutRelationScalars);
            }
        }
        if (transformOptions.includeRequiredFields) {
            const required = definitionPropsMap.reduce((filtered, [name, , fieldMetaData])=>{
                if (fieldMetaData.required && fieldMetaData.isScalar && !fieldMetaData.hasDefaultValue) {
                    filtered.push(name);
                }
                return filtered;
            }, []);
            definition.required = required;
        }
        return [
            model.name,
            definition
        ];
    };
}
